{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Use litgen online"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "This page shows an example of conversion of C++ code into C++ bindings code and python stubs.\n",
    "You can open it in an online interactive environment ([mybinder](https://mybinder.org)), where you can edit the code below.\n",
    "\n",
    "1. To launch the online tool, click on the rocket icon at the top right of this page, or use [this link](https://mybinder.org/v2/gh/pthom/litgen/main?urlpath=lab/tree/litgen-book/01_05_05_online.ipynb)\n",
    "   \n",
    "2. The interactive environment will open in a new tab. Wait for it to be ready\n",
    "   \n",
    "3. Once it is launched, click on \"Run Cell\" to run each of the cells: \n",
    "   \n",
    "  ![Run cell](images/run_cell.jpg)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "You can edit the C++ code as well as the options below. The generated code will be displayed under the cell, once you clicked \"Run Cell\""
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       ".collapsible_header {\n",
       "  background-color: #AAAAAA;\n",
       "  color: white;\n",
       "  cursor: pointer;\n",
       "  padding: 3px;\n",
       "  width: 100%;\n",
       "  border: none;\n",
       "  text-align: left;\n",
       "  outline: none;\n",
       "  font-style: italic;\n",
       "}\n",
       "\n",
       ".collapsible_header_opened {\n",
       "  background-color: #555;\n",
       "}\n",
       "\n",
       ".collapsible_header:hover {\n",
       "  background-color: #AAAAFF;\n",
       "}\n",
       "\n",
       ".collapsible_header:after {\n",
       "  content: '\\002B';\n",
       "  color: white;\n",
       "  font-weight: bold;\n",
       "  float: right;\n",
       "  margin-left: 5px;\n",
       "}\n",
       "\n",
       ".collapsible_header_opened:after {\n",
       "  content: \"\\2212\";\n",
       "}\n",
       "\n",
       ".collapsible_content {\n",
       "  padding: 0 18px;\n",
       "  max-height: 0;\n",
       "  overflow-x: scroll;\n",
       "  overflow-y: scroll;\n",
       "  min-width: 100%;\n",
       "  transition: max-height 0.2s ease-out;\n",
       "  background-color: #f1f1f1;\n",
       "}\n",
       "</style>\n",
       "\n",
       "    <script>\n",
       "       function copy_code_1706190880943_287() {\n",
       "            let code = `####################    <generated_from:BoxedTypes>    ####################\n",
       "class BoxedBool:\n",
       "    value: bool\n",
       "    def __init__(self, v: bool = False) -> None:\n",
       "        pass\n",
       "    def __repr__(self) -> str:\n",
       "        pass\n",
       "####################    </generated_from:BoxedTypes>    ####################\n",
       "\n",
       "\n",
       "\n",
       "def multiply_doubles(v: List[float], k: float) -> List[float]:\n",
       "    \"\"\" Multiplies a list of double by a given factor, returns updated list\"\"\"\n",
       "    pass\n",
       "\n",
       "def switch_bool_value(v: BoxedBool) -> None:\n",
       "    \"\"\" changes the value of the bool parameter (passed by modifiable reference)\n",
       "     (This function will use a BoxedBool in the python code, so that its value can be modified)\n",
       "    \"\"\"\n",
       "    pass\n",
       "\n",
       "# Standalone comment blocs are also exported\n",
       "\n",
       "# This function includes an API marker which will be ignored when generating the bindings\n",
       "def my_substract(a: int, b: int) -> MY_API int:\n",
       "    \"\"\" eol doc is also included in bindings doc\"\"\"\n",
       "    pass\n",
       "\n",
       "\n",
       "class Pixel:\n",
       "    \"\"\" Stores Pixel coordinates\"\"\"\n",
       "    # Coordinates\n",
       "    x: float = 2.\n",
       "    # Coordinates\n",
       "    y: float = 3.\n",
       "\n",
       "    def draw(self, i: Image) -> None:\n",
       "        \"\"\" Draw a pixel\"\"\"\n",
       "        pass\n",
       "\n",
       "    def __init__(self, x: float = 2., y: float = 3.) -> None:\n",
       "        \"\"\"Auto-generated default constructor with named params\"\"\"\n",
       "        pass\n",
       "\n",
       "# This macro value be exported, as it matches the regex macro_define_include_by_name__regex\n",
       "MY_VALUE = 123\n",
       "\n",
       "\n",
       "# <submodule my_math>\n",
       "class my_math:  # Proxy class that introduces typings for the *submodule* my_math\n",
       "    pass  # (This corresponds to a C++ namespace. All method are static!)\n",
       "    \"\"\"This namespace is not ignored and introduces a new python module\"\"\"\n",
       "    # div and mul: divide or multiply float numbers\n",
       "    # (This comment concerns the two grouped\n",
       "    # functions below, and will be exported as such)\n",
       "    @staticmethod\n",
       "    def div(a: float, b: float) -> float:\n",
       "        \"\"\" Divide\"\"\"\n",
       "        pass\n",
       "    @staticmethod\n",
       "    def mul(a: float, b: float) -> float:\n",
       "        \"\"\" Multiply\"\"\"\n",
       "        pass\n",
       "\n",
       "# </submodule my_math>\n",
       "`;\n",
       "            navigator.clipboard.writeText(code);\n",
       "       }\n",
       "    </script>\n",
       "    <button class=\"collapsible_header\" id=\"btn_1706190880943_287_2\" >Corresponding python decls (stub)</button>\n",
       "    <div class=\"collapsible_content\" id=\"content_1706190880943_287_2\">\n",
       "        <div>\n",
       "                <button onclick=\"copy_code_1706190880943_287()\" align=\"right\">copy &#x270d;</button>\n",
       "        </div>\n",
       "        <style>pre { line-height: 125%; }\n",
       "td.linenos .normal { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }\n",
       "span.linenos { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }\n",
       "td.linenos .special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }\n",
       "span.linenos.special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }\n",
       ".highlight .hll { background-color: #3c3836 }\n",
       ".highlight { background: #fbf1c7; }\n",
       ".highlight .c { color: #928374; font-style: italic } /* Comment */\n",
       ".highlight .err { color: #fbf1c7; background-color: #9d0006 } /* Error */\n",
       ".highlight .k { color: #9d0006 } /* Keyword */\n",
       ".highlight .ch { color: #928374; font-style: italic } /* Comment.Hashbang */\n",
       ".highlight .cm { color: #928374; font-style: italic } /* Comment.Multiline */\n",
       ".highlight .c-PreProc { color: #427b58; font-style: italic } /* Comment.PreProc */\n",
       ".highlight .cp { color: #928374; font-style: italic } /* Comment.Preproc */\n",
       ".highlight .cpf { color: #928374; font-style: italic } /* Comment.PreprocFile */\n",
       ".highlight .c1 { color: #928374; font-style: italic } /* Comment.Single */\n",
       ".highlight .cs { color: #3c3836; font-weight: bold; font-style: italic } /* Comment.Special */\n",
       ".highlight .gd { color: #fbf1c7; background-color: #9d0006 } /* Generic.Deleted */\n",
       ".highlight .ge { font-style: italic } /* Generic.Emph */\n",
       ".highlight .gr { color: #9d0006 } /* Generic.Error */\n",
       ".highlight .gh { color: #3c3836; font-weight: bold } /* Generic.Heading */\n",
       ".highlight .gi { color: #fbf1c7; background-color: #79740e } /* Generic.Inserted */\n",
       ".highlight .go { color: #32302f } /* Generic.Output */\n",
       ".highlight .gp { color: #7c6f64 } /* Generic.Prompt */\n",
       ".highlight .gs { font-weight: bold } /* Generic.Strong */\n",
       ".highlight .gu { color: #3c3836; text-decoration: underline } /* Generic.Subheading */\n",
       ".highlight .gt { color: #9d0006 } /* Generic.Traceback */\n",
       ".highlight .kc { color: #9d0006 } /* Keyword.Constant */\n",
       ".highlight .kd { color: #9d0006 } /* Keyword.Declaration */\n",
       ".highlight .kn { color: #9d0006 } /* Keyword.Namespace */\n",
       ".highlight .kp { color: #9d0006 } /* Keyword.Pseudo */\n",
       ".highlight .kr { color: #9d0006 } /* Keyword.Reserved */\n",
       ".highlight .kt { color: #9d0006 } /* Keyword.Type */\n",
       ".highlight .m { color: #8f3f71 } /* Literal.Number */\n",
       ".highlight .s { color: #79740e } /* Literal.String */\n",
       ".highlight .na { color: #b57614 } /* Name.Attribute */\n",
       ".highlight .nb { color: #af3a03 } /* Name.Builtin */\n",
       ".highlight .nc { color: #427b58 } /* Name.Class */\n",
       ".highlight .no { color: #8f3f71 } /* Name.Constant */\n",
       ".highlight .nd { color: #9d0006 } /* Name.Decorator */\n",
       ".highlight .ne { color: #9d0006 } /* Name.Exception */\n",
       ".highlight .nf { color: #427b58 } /* Name.Function */\n",
       ".highlight .nn { color: #427b58 } /* Name.Namespace */\n",
       ".highlight .nt { color: #427b58 } /* Name.Tag */\n",
       ".highlight .nv { color: #076678 } /* Name.Variable */\n",
       ".highlight .ow { color: #9d0006 } /* Operator.Word */\n",
       ".highlight .mb { color: #8f3f71 } /* Literal.Number.Bin */\n",
       ".highlight .mf { color: #8f3f71 } /* Literal.Number.Float */\n",
       ".highlight .mh { color: #8f3f71 } /* Literal.Number.Hex */\n",
       ".highlight .mi { color: #8f3f71 } /* Literal.Number.Integer */\n",
       ".highlight .mo { color: #8f3f71 } /* Literal.Number.Oct */\n",
       ".highlight .sa { color: #79740e } /* Literal.String.Affix */\n",
       ".highlight .sb { color: #79740e } /* Literal.String.Backtick */\n",
       ".highlight .sc { color: #79740e } /* Literal.String.Char */\n",
       ".highlight .dl { color: #79740e } /* Literal.String.Delimiter */\n",
       ".highlight .sd { color: #79740e } /* Literal.String.Doc */\n",
       ".highlight .s2 { color: #79740e } /* Literal.String.Double */\n",
       ".highlight .se { color: #af3a03 } /* Literal.String.Escape */\n",
       ".highlight .sh { color: #79740e } /* Literal.String.Heredoc */\n",
       ".highlight .si { color: #79740e } /* Literal.String.Interpol */\n",
       ".highlight .sx { color: #79740e } /* Literal.String.Other */\n",
       ".highlight .sr { color: #79740e } /* Literal.String.Regex */\n",
       ".highlight .s1 { color: #79740e } /* Literal.String.Single */\n",
       ".highlight .ss { color: #79740e } /* Literal.String.Symbol */\n",
       ".highlight .bp { color: #af3a03 } /* Name.Builtin.Pseudo */\n",
       ".highlight .fm { color: #427b58 } /* Name.Function.Magic */\n",
       ".highlight .vc { color: #076678 } /* Name.Variable.Class */\n",
       ".highlight .vg { color: #076678 } /* Name.Variable.Global */\n",
       ".highlight .vi { color: #076678 } /* Name.Variable.Instance */\n",
       ".highlight .vm { color: #076678 } /* Name.Variable.Magic */\n",
       ".highlight .il { color: #8f3f71 } /* Literal.Number.Integer.Long */</style> <div class=\"highlight\"><pre><span></span><span class=\"c1\">####################    &lt;generated_from:BoxedTypes&gt;    ####################</span>\n",
       "<span class=\"k\">class</span> <span class=\"nc\">BoxedBool</span><span class=\"p\">:</span>\n",
       "    <span class=\"n\">value</span><span class=\"p\">:</span> <span class=\"nb\">bool</span>\n",
       "    <span class=\"k\">def</span> <span class=\"fm\">__init__</span><span class=\"p\">(</span><span class=\"bp\">self</span><span class=\"p\">,</span> <span class=\"n\">v</span><span class=\"p\">:</span> <span class=\"nb\">bool</span> <span class=\"o\">=</span> <span class=\"kc\">False</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"kc\">None</span><span class=\"p\">:</span>\n",
       "        <span class=\"k\">pass</span>\n",
       "    <span class=\"k\">def</span> <span class=\"fm\">__repr__</span><span class=\"p\">(</span><span class=\"bp\">self</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"nb\">str</span><span class=\"p\">:</span>\n",
       "        <span class=\"k\">pass</span>\n",
       "<span class=\"c1\">####################    &lt;/generated_from:BoxedTypes&gt;    ####################</span>\n",
       "\n",
       "\n",
       "\n",
       "<span class=\"k\">def</span> <span class=\"nf\">multiply_doubles</span><span class=\"p\">(</span><span class=\"n\">v</span><span class=\"p\">:</span> <span class=\"n\">List</span><span class=\"p\">[</span><span class=\"nb\">float</span><span class=\"p\">],</span> <span class=\"n\">k</span><span class=\"p\">:</span> <span class=\"nb\">float</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"n\">List</span><span class=\"p\">[</span><span class=\"nb\">float</span><span class=\"p\">]:</span>\n",
       "<span class=\"w\">    </span><span class=\"sd\">&quot;&quot;&quot; Multiplies a list of double by a given factor, returns updated list&quot;&quot;&quot;</span>\n",
       "    <span class=\"k\">pass</span>\n",
       "\n",
       "<span class=\"k\">def</span> <span class=\"nf\">switch_bool_value</span><span class=\"p\">(</span><span class=\"n\">v</span><span class=\"p\">:</span> <span class=\"n\">BoxedBool</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"kc\">None</span><span class=\"p\">:</span>\n",
       "<span class=\"w\">    </span><span class=\"sd\">&quot;&quot;&quot; changes the value of the bool parameter (passed by modifiable reference)</span>\n",
       "<span class=\"sd\">     (This function will use a BoxedBool in the python code, so that its value can be modified)</span>\n",
       "<span class=\"sd\">    &quot;&quot;&quot;</span>\n",
       "    <span class=\"k\">pass</span>\n",
       "\n",
       "<span class=\"c1\"># Standalone comment blocs are also exported</span>\n",
       "\n",
       "<span class=\"c1\"># This function includes an API marker which will be ignored when generating the bindings</span>\n",
       "<span class=\"k\">def</span> <span class=\"nf\">my_substract</span><span class=\"p\">(</span><span class=\"n\">a</span><span class=\"p\">:</span> <span class=\"nb\">int</span><span class=\"p\">,</span> <span class=\"n\">b</span><span class=\"p\">:</span> <span class=\"nb\">int</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"n\">MY_API</span> <span class=\"nb\">int</span><span class=\"p\">:</span>\n",
       "<span class=\"w\">    </span><span class=\"sd\">&quot;&quot;&quot; eol doc is also included in bindings doc&quot;&quot;&quot;</span>\n",
       "    <span class=\"k\">pass</span>\n",
       "\n",
       "\n",
       "<span class=\"k\">class</span> <span class=\"nc\">Pixel</span><span class=\"p\">:</span>\n",
       "<span class=\"w\">    </span><span class=\"sd\">&quot;&quot;&quot; Stores Pixel coordinates&quot;&quot;&quot;</span>\n",
       "    <span class=\"c1\"># Coordinates</span>\n",
       "    <span class=\"n\">x</span><span class=\"p\">:</span> <span class=\"nb\">float</span> <span class=\"o\">=</span> <span class=\"mf\">2.</span>\n",
       "    <span class=\"c1\"># Coordinates</span>\n",
       "    <span class=\"n\">y</span><span class=\"p\">:</span> <span class=\"nb\">float</span> <span class=\"o\">=</span> <span class=\"mf\">3.</span>\n",
       "\n",
       "    <span class=\"k\">def</span> <span class=\"nf\">draw</span><span class=\"p\">(</span><span class=\"bp\">self</span><span class=\"p\">,</span> <span class=\"n\">i</span><span class=\"p\">:</span> <span class=\"n\">Image</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"kc\">None</span><span class=\"p\">:</span>\n",
       "<span class=\"w\">        </span><span class=\"sd\">&quot;&quot;&quot; Draw a pixel&quot;&quot;&quot;</span>\n",
       "        <span class=\"k\">pass</span>\n",
       "\n",
       "    <span class=\"k\">def</span> <span class=\"fm\">__init__</span><span class=\"p\">(</span><span class=\"bp\">self</span><span class=\"p\">,</span> <span class=\"n\">x</span><span class=\"p\">:</span> <span class=\"nb\">float</span> <span class=\"o\">=</span> <span class=\"mf\">2.</span><span class=\"p\">,</span> <span class=\"n\">y</span><span class=\"p\">:</span> <span class=\"nb\">float</span> <span class=\"o\">=</span> <span class=\"mf\">3.</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"kc\">None</span><span class=\"p\">:</span>\n",
       "<span class=\"w\">        </span><span class=\"sd\">&quot;&quot;&quot;Auto-generated default constructor with named params&quot;&quot;&quot;</span>\n",
       "        <span class=\"k\">pass</span>\n",
       "\n",
       "<span class=\"c1\"># This macro value be exported, as it matches the regex macro_define_include_by_name__regex</span>\n",
       "<span class=\"n\">MY_VALUE</span> <span class=\"o\">=</span> <span class=\"mi\">123</span>\n",
       "\n",
       "\n",
       "<span class=\"c1\"># &lt;submodule my_math&gt;</span>\n",
       "<span class=\"k\">class</span> <span class=\"nc\">my_math</span><span class=\"p\">:</span>  <span class=\"c1\"># Proxy class that introduces typings for the *submodule* my_math</span>\n",
       "    <span class=\"k\">pass</span>  <span class=\"c1\"># (This corresponds to a C++ namespace. All method are static!)</span>\n",
       "<span class=\"w\">    </span><span class=\"sd\">&quot;&quot;&quot;This namespace is not ignored and introduces a new python module&quot;&quot;&quot;</span>\n",
       "    <span class=\"c1\"># div and mul: divide or multiply float numbers</span>\n",
       "    <span class=\"c1\"># (This comment concerns the two grouped</span>\n",
       "    <span class=\"c1\"># functions below, and will be exported as such)</span>\n",
       "    <span class=\"nd\">@staticmethod</span>\n",
       "    <span class=\"k\">def</span> <span class=\"nf\">div</span><span class=\"p\">(</span><span class=\"n\">a</span><span class=\"p\">:</span> <span class=\"nb\">float</span><span class=\"p\">,</span> <span class=\"n\">b</span><span class=\"p\">:</span> <span class=\"nb\">float</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"nb\">float</span><span class=\"p\">:</span>\n",
       "<span class=\"w\">        </span><span class=\"sd\">&quot;&quot;&quot; Divide&quot;&quot;&quot;</span>\n",
       "        <span class=\"k\">pass</span>\n",
       "    <span class=\"nd\">@staticmethod</span>\n",
       "    <span class=\"k\">def</span> <span class=\"nf\">mul</span><span class=\"p\">(</span><span class=\"n\">a</span><span class=\"p\">:</span> <span class=\"nb\">float</span><span class=\"p\">,</span> <span class=\"n\">b</span><span class=\"p\">:</span> <span class=\"nb\">float</span><span class=\"p\">)</span> <span class=\"o\">-&gt;</span> <span class=\"nb\">float</span><span class=\"p\">:</span>\n",
       "<span class=\"w\">        </span><span class=\"sd\">&quot;&quot;&quot; Multiply&quot;&quot;&quot;</span>\n",
       "        <span class=\"k\">pass</span>\n",
       "\n",
       "<span class=\"c1\"># &lt;/submodule my_math&gt;</span>\n",
       "</pre></div>\n",
       "\n",
       "    </div>\n",
       "    \n",
       "    <script>\n",
       "    var button = document.getElementById(\"btn_1706190880943_287_2\");\n",
       "    button.addEventListener(\"click\", function() {\n",
       "        this.classList.toggle(\"collapsible_header_opened\");\n",
       "        var content = document.getElementById(\"content_1706190880943_287_2\");\n",
       "        if (content.style.maxHeight){\n",
       "          content.style.maxHeight = null;\n",
       "        } else {\n",
       "          content.style.maxHeight = \"80em\";\n",
       "        }\n",
       "    });\n",
       "    </script>\n",
       "    \n",
       "            <script>\n",
       "            var collapsible_header = document.getElementById(\"btn_1706190880943_287_2\");\n",
       "            collapsible_header.classList.toggle(\"collapsible_header_opened\");\n",
       "            var collapsible_content = document.getElementById(\"content_1706190880943_287_2\");\n",
       "            collapsible_content.style.maxHeight = \"80em\";\n",
       "            </script>\n",
       "            <br/>\n",
       "<style>\n",
       ".collapsible_header {\n",
       "  background-color: #AAAAAA;\n",
       "  color: white;\n",
       "  cursor: pointer;\n",
       "  padding: 3px;\n",
       "  width: 100%;\n",
       "  border: none;\n",
       "  text-align: left;\n",
       "  outline: none;\n",
       "  font-style: italic;\n",
       "}\n",
       "\n",
       ".collapsible_header_opened {\n",
       "  background-color: #555;\n",
       "}\n",
       "\n",
       ".collapsible_header:hover {\n",
       "  background-color: #AAAAFF;\n",
       "}\n",
       "\n",
       ".collapsible_header:after {\n",
       "  content: '\\002B';\n",
       "  color: white;\n",
       "  font-weight: bold;\n",
       "  float: right;\n",
       "  margin-left: 5px;\n",
       "}\n",
       "\n",
       ".collapsible_header_opened:after {\n",
       "  content: \"\\2212\";\n",
       "}\n",
       "\n",
       ".collapsible_content {\n",
       "  padding: 0 18px;\n",
       "  max-height: 0;\n",
       "  overflow-x: scroll;\n",
       "  overflow-y: scroll;\n",
       "  min-width: 100%;\n",
       "  transition: max-height 0.2s ease-out;\n",
       "  background-color: #f1f1f1;\n",
       "}\n",
       "</style>\n",
       "\n",
       "    <script>\n",
       "       function copy_code_1706190880925_5513() {\n",
       "            let code = `////////////////////    <generated_from:BoxedTypes>    ////////////////////\n",
       "auto pyClassBoxedBool =\n",
       "    py::class_<BoxedBool>\n",
       "        (m, \"BoxedBool\", \"\")\n",
       "    .def_readwrite(\"value\", &BoxedBool::value, \"\")\n",
       "    .def(py::init<bool>(),\n",
       "        py::arg(\"v\") = false)\n",
       "    .def(\"__repr__\",\n",
       "        &BoxedBool::__repr__)\n",
       "    ;\n",
       "////////////////////    </generated_from:BoxedTypes>    ////////////////////\n",
       "\n",
       "\n",
       "m.def(\"multiply_doubles\",\n",
       "    MyNamespace::MultiplyDoubles, \n",
       "    py::arg(\"v\"), py::arg(\"k\"), \n",
       "    \"Multiplies a list of double by a given factor, returns updated list\");\n",
       "\n",
       "m.def(\"switch_bool_value\",\n",
       "    [](BoxedBool & v)\n",
       "    {\n",
       "        auto SwitchBoolValue_adapt_modifiable_immutable = [](BoxedBool & v)\n",
       "        {\n",
       "            bool & v_boxed_value = v.value;\n",
       "\n",
       "            MyNamespace::SwitchBoolValue(v_boxed_value);\n",
       "        };\n",
       "\n",
       "        SwitchBoolValue_adapt_modifiable_immutable(v);\n",
       "    }, \n",
       "    py::arg(\"v\"), \n",
       "    \" changes the value of the bool parameter (passed by modifiable reference)\\n (This function will use a BoxedBool in the python code, so that its value can be modified)\");\n",
       "\n",
       "m.def(\"my_substract\",\n",
       "    MyNamespace::MySubstract, \n",
       "    py::arg(\"a\"), py::arg(\"b\"), \n",
       "    \"eol doc is also included in bindings doc\");\n",
       "\n",
       "\n",
       "auto pyClassPixel =\n",
       "    py::class_<MyNamespace::Pixel>\n",
       "        (m, \"Pixel\", \"Stores Pixel coordinates\")\n",
       "    .def(py::init<>([](\n",
       "    double x = 2., double y = 3.)\n",
       "    {\n",
       "        auto r = std::make_unique<MyNamespace::Pixel>();\n",
       "        r->x = x;\n",
       "        r->y = y;\n",
       "        return r;\n",
       "    })\n",
       "    , py::arg(\"x\") = 2., py::arg(\"y\") = 3.\n",
       "    )\n",
       "    .def_readwrite(\"x\", &MyNamespace::Pixel::x, \"Coordinates\")\n",
       "    .def_readwrite(\"y\", &MyNamespace::Pixel::y, \"Coordinates\")\n",
       "    .def(\"draw\",\n",
       "        &MyNamespace::Pixel::Draw,\n",
       "        py::arg(\"i\"),\n",
       "        \"Draw a pixel\")\n",
       "    ;\n",
       "m.attr(\"MY_VALUE\") = 123;\n",
       "\n",
       "{ // <namespace MyMath>\n",
       "    py::module_ pyNsMyMath = m.def_submodule(\"my_math\", \"This namespace is not ignored and introduces a new python module\");\n",
       "    pyNsMyMath.def(\"div\",\n",
       "        MyNamespace::MyMath::Div,\n",
       "        py::arg(\"a\"), py::arg(\"b\"),\n",
       "        \"Divide\");\n",
       "\n",
       "    pyNsMyMath.def(\"mul\",\n",
       "        MyNamespace::MyMath::Mul,\n",
       "        py::arg(\"a\"), py::arg(\"b\"),\n",
       "        \"Multiply\");\n",
       "} // </namespace MyMath>\n",
       "`;\n",
       "            navigator.clipboard.writeText(code);\n",
       "       }\n",
       "    </script>\n",
       "    <button class=\"collapsible_header\" id=\"btn_1706190880925_5513_0\" >pybind11 C++ binding code</button>\n",
       "    <div class=\"collapsible_content\" id=\"content_1706190880925_5513_0\">\n",
       "        <div>\n",
       "                <button onclick=\"copy_code_1706190880925_5513()\" align=\"right\">copy &#x270d;</button>\n",
       "        </div>\n",
       "        <style>pre { line-height: 125%; }\n",
       "td.linenos .normal { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }\n",
       "span.linenos { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }\n",
       "td.linenos .special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }\n",
       "span.linenos.special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }\n",
       ".highlight .hll { background-color: #3c3836 }\n",
       ".highlight { background: #fbf1c7; }\n",
       ".highlight .c { color: #928374; font-style: italic } /* Comment */\n",
       ".highlight .err { color: #fbf1c7; background-color: #9d0006 } /* Error */\n",
       ".highlight .k { color: #9d0006 } /* Keyword */\n",
       ".highlight .ch { color: #928374; font-style: italic } /* Comment.Hashbang */\n",
       ".highlight .cm { color: #928374; font-style: italic } /* Comment.Multiline */\n",
       ".highlight .c-PreProc { color: #427b58; font-style: italic } /* Comment.PreProc */\n",
       ".highlight .cp { color: #928374; font-style: italic } /* Comment.Preproc */\n",
       ".highlight .cpf { color: #928374; font-style: italic } /* Comment.PreprocFile */\n",
       ".highlight .c1 { color: #928374; font-style: italic } /* Comment.Single */\n",
       ".highlight .cs { color: #3c3836; font-weight: bold; font-style: italic } /* Comment.Special */\n",
       ".highlight .gd { color: #fbf1c7; background-color: #9d0006 } /* Generic.Deleted */\n",
       ".highlight .ge { font-style: italic } /* Generic.Emph */\n",
       ".highlight .gr { color: #9d0006 } /* Generic.Error */\n",
       ".highlight .gh { color: #3c3836; font-weight: bold } /* Generic.Heading */\n",
       ".highlight .gi { color: #fbf1c7; background-color: #79740e } /* Generic.Inserted */\n",
       ".highlight .go { color: #32302f } /* Generic.Output */\n",
       ".highlight .gp { color: #7c6f64 } /* Generic.Prompt */\n",
       ".highlight .gs { font-weight: bold } /* Generic.Strong */\n",
       ".highlight .gu { color: #3c3836; text-decoration: underline } /* Generic.Subheading */\n",
       ".highlight .gt { color: #9d0006 } /* Generic.Traceback */\n",
       ".highlight .kc { color: #9d0006 } /* Keyword.Constant */\n",
       ".highlight .kd { color: #9d0006 } /* Keyword.Declaration */\n",
       ".highlight .kn { color: #9d0006 } /* Keyword.Namespace */\n",
       ".highlight .kp { color: #9d0006 } /* Keyword.Pseudo */\n",
       ".highlight .kr { color: #9d0006 } /* Keyword.Reserved */\n",
       ".highlight .kt { color: #9d0006 } /* Keyword.Type */\n",
       ".highlight .m { color: #8f3f71 } /* Literal.Number */\n",
       ".highlight .s { color: #79740e } /* Literal.String */\n",
       ".highlight .na { color: #b57614 } /* Name.Attribute */\n",
       ".highlight .nb { color: #af3a03 } /* Name.Builtin */\n",
       ".highlight .nc { color: #427b58 } /* Name.Class */\n",
       ".highlight .no { color: #8f3f71 } /* Name.Constant */\n",
       ".highlight .nd { color: #9d0006 } /* Name.Decorator */\n",
       ".highlight .ne { color: #9d0006 } /* Name.Exception */\n",
       ".highlight .nf { color: #427b58 } /* Name.Function */\n",
       ".highlight .nn { color: #427b58 } /* Name.Namespace */\n",
       ".highlight .nt { color: #427b58 } /* Name.Tag */\n",
       ".highlight .nv { color: #076678 } /* Name.Variable */\n",
       ".highlight .ow { color: #9d0006 } /* Operator.Word */\n",
       ".highlight .mb { color: #8f3f71 } /* Literal.Number.Bin */\n",
       ".highlight .mf { color: #8f3f71 } /* Literal.Number.Float */\n",
       ".highlight .mh { color: #8f3f71 } /* Literal.Number.Hex */\n",
       ".highlight .mi { color: #8f3f71 } /* Literal.Number.Integer */\n",
       ".highlight .mo { color: #8f3f71 } /* Literal.Number.Oct */\n",
       ".highlight .sa { color: #79740e } /* Literal.String.Affix */\n",
       ".highlight .sb { color: #79740e } /* Literal.String.Backtick */\n",
       ".highlight .sc { color: #79740e } /* Literal.String.Char */\n",
       ".highlight .dl { color: #79740e } /* Literal.String.Delimiter */\n",
       ".highlight .sd { color: #79740e } /* Literal.String.Doc */\n",
       ".highlight .s2 { color: #79740e } /* Literal.String.Double */\n",
       ".highlight .se { color: #af3a03 } /* Literal.String.Escape */\n",
       ".highlight .sh { color: #79740e } /* Literal.String.Heredoc */\n",
       ".highlight .si { color: #79740e } /* Literal.String.Interpol */\n",
       ".highlight .sx { color: #79740e } /* Literal.String.Other */\n",
       ".highlight .sr { color: #79740e } /* Literal.String.Regex */\n",
       ".highlight .s1 { color: #79740e } /* Literal.String.Single */\n",
       ".highlight .ss { color: #79740e } /* Literal.String.Symbol */\n",
       ".highlight .bp { color: #af3a03 } /* Name.Builtin.Pseudo */\n",
       ".highlight .fm { color: #427b58 } /* Name.Function.Magic */\n",
       ".highlight .vc { color: #076678 } /* Name.Variable.Class */\n",
       ".highlight .vg { color: #076678 } /* Name.Variable.Global */\n",
       ".highlight .vi { color: #076678 } /* Name.Variable.Instance */\n",
       ".highlight .vm { color: #076678 } /* Name.Variable.Magic */\n",
       ".highlight .il { color: #8f3f71 } /* Literal.Number.Integer.Long */</style> <div class=\"highlight\"><pre><span></span><span class=\"c1\">////////////////////    &lt;generated_from:BoxedTypes&gt;    ////////////////////</span>\n",
       "<span class=\"k\">auto</span><span class=\"w\"> </span><span class=\"n\">pyClassBoxedBool</span><span class=\"w\"> </span><span class=\"o\">=</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">class_</span><span class=\"o\">&lt;</span><span class=\"n\">BoxedBool</span><span class=\"o\">&gt;</span>\n",
       "<span class=\"w\">        </span><span class=\"p\">(</span><span class=\"n\">m</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;BoxedBool&quot;</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;&quot;</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">.</span><span class=\"n\">def_readwrite</span><span class=\"p\">(</span><span class=\"s\">&quot;value&quot;</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"o\">&amp;</span><span class=\"n\">BoxedBool</span><span class=\"o\">::</span><span class=\"n\">value</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;&quot;</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">init</span><span class=\"o\">&lt;</span><span class=\"kt\">bool</span><span class=\"o\">&gt;</span><span class=\"p\">(),</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;v&quot;</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"nb\">false</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"s\">&quot;__repr__&quot;</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">        </span><span class=\"o\">&amp;</span><span class=\"n\">BoxedBool</span><span class=\"o\">::</span><span class=\"n\">__repr__</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">;</span>\n",
       "<span class=\"c1\">////////////////////    &lt;/generated_from:BoxedTypes&gt;    ////////////////////</span>\n",
       "\n",
       "\n",
       "<span class=\"n\">m</span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"s\">&quot;multiply_doubles&quot;</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">MultiplyDoubles</span><span class=\"p\">,</span><span class=\"w\"> </span>\n",
       "<span class=\"w\">    </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;v&quot;</span><span class=\"p\">),</span><span class=\"w\"> </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;k&quot;</span><span class=\"p\">),</span><span class=\"w\"> </span>\n",
       "<span class=\"w\">    </span><span class=\"s\">&quot;Multiplies a list of double by a given factor, returns updated list&quot;</span><span class=\"p\">);</span>\n",
       "\n",
       "<span class=\"n\">m</span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"s\">&quot;switch_bool_value&quot;</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">[](</span><span class=\"n\">BoxedBool</span><span class=\"w\"> </span><span class=\"o\">&amp;</span><span class=\"w\"> </span><span class=\"n\">v</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">{</span>\n",
       "<span class=\"w\">        </span><span class=\"k\">auto</span><span class=\"w\"> </span><span class=\"n\">SwitchBoolValue_adapt_modifiable_immutable</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"p\">[](</span><span class=\"n\">BoxedBool</span><span class=\"w\"> </span><span class=\"o\">&amp;</span><span class=\"w\"> </span><span class=\"n\">v</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">        </span><span class=\"p\">{</span>\n",
       "<span class=\"w\">            </span><span class=\"kt\">bool</span><span class=\"w\"> </span><span class=\"o\">&amp;</span><span class=\"w\"> </span><span class=\"n\">v_boxed_value</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"n\">v</span><span class=\"p\">.</span><span class=\"n\">value</span><span class=\"p\">;</span>\n",
       "\n",
       "<span class=\"w\">            </span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">SwitchBoolValue</span><span class=\"p\">(</span><span class=\"n\">v_boxed_value</span><span class=\"p\">);</span>\n",
       "<span class=\"w\">        </span><span class=\"p\">};</span>\n",
       "\n",
       "<span class=\"w\">        </span><span class=\"n\">SwitchBoolValue_adapt_modifiable_immutable</span><span class=\"p\">(</span><span class=\"n\">v</span><span class=\"p\">);</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">},</span><span class=\"w\"> </span>\n",
       "<span class=\"w\">    </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;v&quot;</span><span class=\"p\">),</span><span class=\"w\"> </span>\n",
       "<span class=\"w\">    </span><span class=\"s\">&quot; changes the value of the bool parameter (passed by modifiable reference)</span><span class=\"se\">\\n</span><span class=\"s\"> (This function will use a BoxedBool in the python code, so that its value can be modified)&quot;</span><span class=\"p\">);</span>\n",
       "\n",
       "<span class=\"n\">m</span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"s\">&quot;my_substract&quot;</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">MySubstract</span><span class=\"p\">,</span><span class=\"w\"> </span>\n",
       "<span class=\"w\">    </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;a&quot;</span><span class=\"p\">),</span><span class=\"w\"> </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;b&quot;</span><span class=\"p\">),</span><span class=\"w\"> </span>\n",
       "<span class=\"w\">    </span><span class=\"s\">&quot;eol doc is also included in bindings doc&quot;</span><span class=\"p\">);</span>\n",
       "\n",
       "\n",
       "<span class=\"k\">auto</span><span class=\"w\"> </span><span class=\"n\">pyClassPixel</span><span class=\"w\"> </span><span class=\"o\">=</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">class_</span><span class=\"o\">&lt;</span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">Pixel</span><span class=\"o\">&gt;</span>\n",
       "<span class=\"w\">        </span><span class=\"p\">(</span><span class=\"n\">m</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;Pixel&quot;</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;Stores Pixel coordinates&quot;</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">init</span><span class=\"o\">&lt;&gt;</span><span class=\"p\">([](</span>\n",
       "<span class=\"w\">    </span><span class=\"kt\">double</span><span class=\"w\"> </span><span class=\"n\">x</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"mf\">2.</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"kt\">double</span><span class=\"w\"> </span><span class=\"n\">y</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"mf\">3.</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">{</span>\n",
       "<span class=\"w\">        </span><span class=\"k\">auto</span><span class=\"w\"> </span><span class=\"n\">r</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"n\">std</span><span class=\"o\">::</span><span class=\"n\">make_unique</span><span class=\"o\">&lt;</span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">Pixel</span><span class=\"o\">&gt;</span><span class=\"p\">();</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">r</span><span class=\"o\">-&gt;</span><span class=\"n\">x</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"n\">x</span><span class=\"p\">;</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">r</span><span class=\"o\">-&gt;</span><span class=\"n\">y</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"n\">y</span><span class=\"p\">;</span>\n",
       "<span class=\"w\">        </span><span class=\"k\">return</span><span class=\"w\"> </span><span class=\"n\">r</span><span class=\"p\">;</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">})</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;x&quot;</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"mf\">2.</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;y&quot;</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"mf\">3.</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">.</span><span class=\"n\">def_readwrite</span><span class=\"p\">(</span><span class=\"s\">&quot;x&quot;</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"o\">&amp;</span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">Pixel</span><span class=\"o\">::</span><span class=\"n\">x</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;Coordinates&quot;</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">.</span><span class=\"n\">def_readwrite</span><span class=\"p\">(</span><span class=\"s\">&quot;y&quot;</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"o\">&amp;</span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">Pixel</span><span class=\"o\">::</span><span class=\"n\">y</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;Coordinates&quot;</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"s\">&quot;draw&quot;</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">        </span><span class=\"o\">&amp;</span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">Pixel</span><span class=\"o\">::</span><span class=\"n\">Draw</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;i&quot;</span><span class=\"p\">),</span>\n",
       "<span class=\"w\">        </span><span class=\"s\">&quot;Draw a pixel&quot;</span><span class=\"p\">)</span>\n",
       "<span class=\"w\">    </span><span class=\"p\">;</span>\n",
       "<span class=\"n\">m</span><span class=\"p\">.</span><span class=\"n\">attr</span><span class=\"p\">(</span><span class=\"s\">&quot;MY_VALUE&quot;</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"mi\">123</span><span class=\"p\">;</span>\n",
       "\n",
       "<span class=\"p\">{</span><span class=\"w\"> </span><span class=\"c1\">// &lt;namespace MyMath&gt;</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">module_</span><span class=\"w\"> </span><span class=\"n\">pyNsMyMath</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"n\">m</span><span class=\"p\">.</span><span class=\"n\">def_submodule</span><span class=\"p\">(</span><span class=\"s\">&quot;my_math&quot;</span><span class=\"p\">,</span><span class=\"w\"> </span><span class=\"s\">&quot;This namespace is not ignored and introduces a new python module&quot;</span><span class=\"p\">);</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">pyNsMyMath</span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"s\">&quot;div&quot;</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">MyMath</span><span class=\"o\">::</span><span class=\"n\">Div</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;a&quot;</span><span class=\"p\">),</span><span class=\"w\"> </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;b&quot;</span><span class=\"p\">),</span>\n",
       "<span class=\"w\">        </span><span class=\"s\">&quot;Divide&quot;</span><span class=\"p\">);</span>\n",
       "\n",
       "<span class=\"w\">    </span><span class=\"n\">pyNsMyMath</span><span class=\"p\">.</span><span class=\"n\">def</span><span class=\"p\">(</span><span class=\"s\">&quot;mul&quot;</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">MyNamespace</span><span class=\"o\">::</span><span class=\"n\">MyMath</span><span class=\"o\">::</span><span class=\"n\">Mul</span><span class=\"p\">,</span>\n",
       "<span class=\"w\">        </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;a&quot;</span><span class=\"p\">),</span><span class=\"w\"> </span><span class=\"n\">py</span><span class=\"o\">::</span><span class=\"n\">arg</span><span class=\"p\">(</span><span class=\"s\">&quot;b&quot;</span><span class=\"p\">),</span>\n",
       "<span class=\"w\">        </span><span class=\"s\">&quot;Multiply&quot;</span><span class=\"p\">);</span>\n",
       "<span class=\"p\">}</span><span class=\"w\"> </span><span class=\"c1\">// &lt;/namespace MyMath&gt;</span>\n",
       "</pre></div>\n",
       "\n",
       "    </div>\n",
       "    \n",
       "    <script>\n",
       "    var button = document.getElementById(\"btn_1706190880925_5513_0\");\n",
       "    button.addEventListener(\"click\", function() {\n",
       "        this.classList.toggle(\"collapsible_header_opened\");\n",
       "        var content = document.getElementById(\"content_1706190880925_5513_0\");\n",
       "        if (content.style.maxHeight){\n",
       "          content.style.maxHeight = null;\n",
       "        } else {\n",
       "          content.style.maxHeight = \"80em\";\n",
       "        }\n",
       "    });\n",
       "    </script>\n",
       "    \n",
       "            <script>\n",
       "            var collapsible_header = document.getElementById(\"btn_1706190880925_5513_0\");\n",
       "            collapsible_header.classList.toggle(\"collapsible_header_opened\");\n",
       "            var collapsible_content = document.getElementById(\"content_1706190880925_5513_0\");\n",
       "            collapsible_content.style.maxHeight = \"80em\";\n",
       "            </script>\n",
       "            <br/>\n",
       "<style>\n",
       ".collapsible_header {\n",
       "  background-color: #AAAAAA;\n",
       "  color: white;\n",
       "  cursor: pointer;\n",
       "  padding: 3px;\n",
       "  width: 100%;\n",
       "  border: none;\n",
       "  text-align: left;\n",
       "  outline: none;\n",
       "  font-style: italic;\n",
       "}\n",
       "\n",
       ".collapsible_header_opened {\n",
       "  background-color: #555;\n",
       "}\n",
       "\n",
       ".collapsible_header:hover {\n",
       "  background-color: #AAAAFF;\n",
       "}\n",
       "\n",
       ".collapsible_header:after {\n",
       "  content: '\\002B';\n",
       "  color: white;\n",
       "  font-weight: bold;\n",
       "  float: right;\n",
       "  margin-left: 5px;\n",
       "}\n",
       "\n",
       ".collapsible_header_opened:after {\n",
       "  content: \"\\2212\";\n",
       "}\n",
       "\n",
       ".collapsible_content {\n",
       "  padding: 0 18px;\n",
       "  max-height: 0;\n",
       "  overflow-x: scroll;\n",
       "  overflow-y: scroll;\n",
       "  min-width: 100%;\n",
       "  transition: max-height 0.2s ease-out;\n",
       "  background-color: #f1f1f1;\n",
       "}\n",
       "</style>\n",
       "\n",
       "    <script>\n",
       "       function copy_code_1706190880926_136() {\n",
       "            let code = `struct BoxedBool\n",
       "{\n",
       "    bool value;\n",
       "    BoxedBool(bool v = false) : value(v) {}\n",
       "    std::string __repr__() const { return std::string(\"BoxedBool(\") + std::to_string(value) + \")\"; }\n",
       "};\n",
       "`;\n",
       "            navigator.clipboard.writeText(code);\n",
       "       }\n",
       "    </script>\n",
       "    <button class=\"collapsible_header\" id=\"btn_1706190880926_136_1\" >C++ glue code</button>\n",
       "    <div class=\"collapsible_content\" id=\"content_1706190880926_136_1\">\n",
       "        <div>\n",
       "                <button onclick=\"copy_code_1706190880926_136()\" align=\"right\">copy &#x270d;</button>\n",
       "        </div>\n",
       "        <style>pre { line-height: 125%; }\n",
       "td.linenos .normal { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }\n",
       "span.linenos { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }\n",
       "td.linenos .special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }\n",
       "span.linenos.special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }\n",
       ".highlight .hll { background-color: #3c3836 }\n",
       ".highlight { background: #fbf1c7; }\n",
       ".highlight .c { color: #928374; font-style: italic } /* Comment */\n",
       ".highlight .err { color: #fbf1c7; background-color: #9d0006 } /* Error */\n",
       ".highlight .k { color: #9d0006 } /* Keyword */\n",
       ".highlight .ch { color: #928374; font-style: italic } /* Comment.Hashbang */\n",
       ".highlight .cm { color: #928374; font-style: italic } /* Comment.Multiline */\n",
       ".highlight .c-PreProc { color: #427b58; font-style: italic } /* Comment.PreProc */\n",
       ".highlight .cp { color: #928374; font-style: italic } /* Comment.Preproc */\n",
       ".highlight .cpf { color: #928374; font-style: italic } /* Comment.PreprocFile */\n",
       ".highlight .c1 { color: #928374; font-style: italic } /* Comment.Single */\n",
       ".highlight .cs { color: #3c3836; font-weight: bold; font-style: italic } /* Comment.Special */\n",
       ".highlight .gd { color: #fbf1c7; background-color: #9d0006 } /* Generic.Deleted */\n",
       ".highlight .ge { font-style: italic } /* Generic.Emph */\n",
       ".highlight .gr { color: #9d0006 } /* Generic.Error */\n",
       ".highlight .gh { color: #3c3836; font-weight: bold } /* Generic.Heading */\n",
       ".highlight .gi { color: #fbf1c7; background-color: #79740e } /* Generic.Inserted */\n",
       ".highlight .go { color: #32302f } /* Generic.Output */\n",
       ".highlight .gp { color: #7c6f64 } /* Generic.Prompt */\n",
       ".highlight .gs { font-weight: bold } /* Generic.Strong */\n",
       ".highlight .gu { color: #3c3836; text-decoration: underline } /* Generic.Subheading */\n",
       ".highlight .gt { color: #9d0006 } /* Generic.Traceback */\n",
       ".highlight .kc { color: #9d0006 } /* Keyword.Constant */\n",
       ".highlight .kd { color: #9d0006 } /* Keyword.Declaration */\n",
       ".highlight .kn { color: #9d0006 } /* Keyword.Namespace */\n",
       ".highlight .kp { color: #9d0006 } /* Keyword.Pseudo */\n",
       ".highlight .kr { color: #9d0006 } /* Keyword.Reserved */\n",
       ".highlight .kt { color: #9d0006 } /* Keyword.Type */\n",
       ".highlight .m { color: #8f3f71 } /* Literal.Number */\n",
       ".highlight .s { color: #79740e } /* Literal.String */\n",
       ".highlight .na { color: #b57614 } /* Name.Attribute */\n",
       ".highlight .nb { color: #af3a03 } /* Name.Builtin */\n",
       ".highlight .nc { color: #427b58 } /* Name.Class */\n",
       ".highlight .no { color: #8f3f71 } /* Name.Constant */\n",
       ".highlight .nd { color: #9d0006 } /* Name.Decorator */\n",
       ".highlight .ne { color: #9d0006 } /* Name.Exception */\n",
       ".highlight .nf { color: #427b58 } /* Name.Function */\n",
       ".highlight .nn { color: #427b58 } /* Name.Namespace */\n",
       ".highlight .nt { color: #427b58 } /* Name.Tag */\n",
       ".highlight .nv { color: #076678 } /* Name.Variable */\n",
       ".highlight .ow { color: #9d0006 } /* Operator.Word */\n",
       ".highlight .mb { color: #8f3f71 } /* Literal.Number.Bin */\n",
       ".highlight .mf { color: #8f3f71 } /* Literal.Number.Float */\n",
       ".highlight .mh { color: #8f3f71 } /* Literal.Number.Hex */\n",
       ".highlight .mi { color: #8f3f71 } /* Literal.Number.Integer */\n",
       ".highlight .mo { color: #8f3f71 } /* Literal.Number.Oct */\n",
       ".highlight .sa { color: #79740e } /* Literal.String.Affix */\n",
       ".highlight .sb { color: #79740e } /* Literal.String.Backtick */\n",
       ".highlight .sc { color: #79740e } /* Literal.String.Char */\n",
       ".highlight .dl { color: #79740e } /* Literal.String.Delimiter */\n",
       ".highlight .sd { color: #79740e } /* Literal.String.Doc */\n",
       ".highlight .s2 { color: #79740e } /* Literal.String.Double */\n",
       ".highlight .se { color: #af3a03 } /* Literal.String.Escape */\n",
       ".highlight .sh { color: #79740e } /* Literal.String.Heredoc */\n",
       ".highlight .si { color: #79740e } /* Literal.String.Interpol */\n",
       ".highlight .sx { color: #79740e } /* Literal.String.Other */\n",
       ".highlight .sr { color: #79740e } /* Literal.String.Regex */\n",
       ".highlight .s1 { color: #79740e } /* Literal.String.Single */\n",
       ".highlight .ss { color: #79740e } /* Literal.String.Symbol */\n",
       ".highlight .bp { color: #af3a03 } /* Name.Builtin.Pseudo */\n",
       ".highlight .fm { color: #427b58 } /* Name.Function.Magic */\n",
       ".highlight .vc { color: #076678 } /* Name.Variable.Class */\n",
       ".highlight .vg { color: #076678 } /* Name.Variable.Global */\n",
       ".highlight .vi { color: #076678 } /* Name.Variable.Instance */\n",
       ".highlight .vm { color: #076678 } /* Name.Variable.Magic */\n",
       ".highlight .il { color: #8f3f71 } /* Literal.Number.Integer.Long */</style> <div class=\"highlight\"><pre><span></span><span class=\"k\">struct</span><span class=\"w\"> </span><span class=\"nc\">BoxedBool</span>\n",
       "<span class=\"p\">{</span>\n",
       "<span class=\"w\">    </span><span class=\"kt\">bool</span><span class=\"w\"> </span><span class=\"n\">value</span><span class=\"p\">;</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">BoxedBool</span><span class=\"p\">(</span><span class=\"kt\">bool</span><span class=\"w\"> </span><span class=\"n\">v</span><span class=\"w\"> </span><span class=\"o\">=</span><span class=\"w\"> </span><span class=\"nb\">false</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"o\">:</span><span class=\"w\"> </span><span class=\"n\">value</span><span class=\"p\">(</span><span class=\"n\">v</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"p\">{}</span>\n",
       "<span class=\"w\">    </span><span class=\"n\">std</span><span class=\"o\">::</span><span class=\"n\">string</span><span class=\"w\"> </span><span class=\"n\">__repr__</span><span class=\"p\">()</span><span class=\"w\"> </span><span class=\"k\">const</span><span class=\"w\"> </span><span class=\"p\">{</span><span class=\"w\"> </span><span class=\"k\">return</span><span class=\"w\"> </span><span class=\"n\">std</span><span class=\"o\">::</span><span class=\"n\">string</span><span class=\"p\">(</span><span class=\"s\">&quot;BoxedBool(&quot;</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"o\">+</span><span class=\"w\"> </span><span class=\"n\">std</span><span class=\"o\">::</span><span class=\"n\">to_string</span><span class=\"p\">(</span><span class=\"n\">value</span><span class=\"p\">)</span><span class=\"w\"> </span><span class=\"o\">+</span><span class=\"w\"> </span><span class=\"s\">&quot;)&quot;</span><span class=\"p\">;</span><span class=\"w\"> </span><span class=\"p\">}</span>\n",
       "<span class=\"p\">};</span>\n",
       "</pre></div>\n",
       "\n",
       "    </div>\n",
       "    \n",
       "    <script>\n",
       "    var button = document.getElementById(\"btn_1706190880926_136_1\");\n",
       "    button.addEventListener(\"click\", function() {\n",
       "        this.classList.toggle(\"collapsible_header_opened\");\n",
       "        var content = document.getElementById(\"content_1706190880926_136_1\");\n",
       "        if (content.style.maxHeight){\n",
       "          content.style.maxHeight = null;\n",
       "        } else {\n",
       "          content.style.maxHeight = \"80em\";\n",
       "        }\n",
       "    });\n",
       "    </script>\n",
       "    \n",
       "            <script>\n",
       "            var collapsible_header = document.getElementById(\"btn_1706190880926_136_1\");\n",
       "            collapsible_header.classList.toggle(\"collapsible_header_opened\");\n",
       "            var collapsible_content = document.getElementById(\"content_1706190880926_136_1\");\n",
       "            collapsible_content.style.maxHeight = \"80em\";\n",
       "            </script>\n",
       "            "
      ],
      "text/plain": [
       "<IPython.core.display.HTML object>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "import litgen\n",
    "from litgen.demo import litgen_demo\n",
    "\n",
    "options = litgen.LitgenOptions()\n",
    "\n",
    "# This namespace will not be outputted as a python module\n",
    "options.namespaces_root = [\"MyNamespace\"]\n",
    "# All functions, modules and namespaces names are converted to snake case\n",
    "options.python_convert_to_snake_case = True\n",
    "# This is an API marker in the C++ code (for shared libraries code)\n",
    "options.srcmlcpp_options.functions_api_prefixes = \"^MY_API$\"\n",
    "# Also create bindings for functions that do not have the API marker\n",
    "options.fn_exclude_non_api = False\n",
    "# Optional comment that can be added to non API functions\n",
    "options.fn_non_api_comment = \"\"\n",
    "# \"Box\" immutable functions parameter when they should be modifiable\n",
    "options.fn_params_replace_modifiable_immutable_by_boxed__regex = r\".*\"\n",
    "# Which numeric and string preprocessor do we want to export\n",
    "options.macro_define_include_by_name__regex = \"^MY_\"\n",
    "\n",
    "code = \"\"\"\n",
    "// This namespace is not outputed as a submodule, since it is marked as Root (see options.namespaces_root)\n",
    "namespace MyNamespace \n",
    "{\n",
    "    // Multiplies a list of double by a given factor, returns updated list\n",
    "    std::vector<double> MultiplyDoubles(const std::vector<double>& v, float k);\n",
    "\n",
    "    // changes the value of the bool parameter (passed by modifiable reference)\n",
    "    // (This function will use a BoxedBool in the python code, so that its value can be modified)\n",
    "    void SwitchBoolValue(bool &v);\n",
    "    \n",
    "    // Standalone comment blocs are also exported\n",
    "\n",
    "    // This function includes an API marker which will be ignored when generating the bindings\n",
    "    MY_API int MySubstract(int a, int b); // eol doc is also included in bindings doc\n",
    "\n",
    "    namespace MyMath // This namespace is not ignored and introduces a new python module\n",
    "    {\n",
    "        // div and mul: divide or multiply float numbers\n",
    "        // (This comment concerns the two grouped \n",
    "        // functions below, and will be exported as such)\n",
    "        float Div(float a, float b); // Divide\n",
    "        float Mul(float a, float b); // Multiply\n",
    "    }\n",
    "\n",
    "    // Stores Pixel coordinates\n",
    "    struct Pixel\n",
    "    {\n",
    "        // Coordinates\n",
    "        double x = 2., y = 3.;\n",
    "\n",
    "        // Draw a pixel\n",
    "        void Draw(Image& i);\n",
    "\n",
    "        private:\n",
    "        double _Norm(); // this will not be exported as it is private\n",
    "    };\n",
    "\n",
    "    // This macro value be exported, as it matches the regex macro_define_include_by_name__regex\n",
    "    #define MY_VALUE 123\n",
    "}\n",
    "\"\"\"\n",
    "\n",
    "litgen_demo.demo(\n",
    "    options, code, show_cpp=False, show_pydef=True, height=80\n",
    ")"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.0"
  },
  "toc": {
   "base_numbering": 1,
   "nav_menu": {},
   "number_sections": true,
   "sideBar": true,
   "skip_h1_title": false,
   "title_cell": "Table of Contents",
   "title_sidebar": "Contents",
   "toc_cell": false,
   "toc_position": {
    "height": "calc(100% - 180px)",
    "left": "10px",
    "top": "150px",
    "width": "345.6px"
   },
   "toc_section_display": true,
   "toc_window_display": true
  },
  "varInspector": {
   "cols": {
    "lenName": 16,
    "lenType": 16,
    "lenVar": 40
   },
   "kernels_config": {
    "python": {
     "delete_cmd_postfix": "",
     "delete_cmd_prefix": "del ",
     "library": "var_list.py",
     "varRefreshCmd": "print(var_dic_list())"
    },
    "r": {
     "delete_cmd_postfix": ") ",
     "delete_cmd_prefix": "rm(",
     "library": "var_list.r",
     "varRefreshCmd": "cat(var_dic_list()) "
    }
   },
   "oldHeight": 336.35,
   "position": {
    "height": "358.85px",
    "left": "1341.5px",
    "right": "20px",
    "top": "120px",
    "width": "358.5px"
   },
   "types_to_exclude": [
    "module",
    "function",
    "builtin_function_or_method",
    "instance",
    "_Feature"
   ],
   "varInspector_section_display": "block",
   "window_display": false
  },
  "vscode": {
   "interpreter": {
    "hash": "3cc45f2ea8bd08db24aa5fe847d95c7113ff50f58968fda0dbbcaf292fcbf0db"
   }
  },
  "widgets": {
   "application/vnd.jupyter.widget-state+json": {
    "state": {},
    "version_major": 2,
    "version_minor": 0
   }
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
